"use strict";
Object.defineProperty(exports, "default", {
    enumerable: true,
    get: ()=>_default
});
const _exec = require("@actions/exec");
const _devkit = require("@nrwl/devkit");
const _pluginConfig = require("../../utils/plugin-config");
const _os = require("os");
const _postTargets = require("../../utils/post-targets");
const fs = require("fs/promises");
const path = require("path");
const runExecutor = async (options, context)=>{
    const { vercelCommand ='vercel'  } = (0, _pluginConfig.getPluginConfig)(context);
    if (!context.projectName) {
        throw new Error('nx-vercel requires projectName to be assigned');
    }
    const projectPath = path.resolve(context.root, context.workspace.projects[context.projectName].root);
    const vercelDir = path.resolve(projectPath, '.vercel');
    const rootVercelDir = path.resolve(context.root, '.vercel');
    const buildArgs = [
        'build'
    ];
    const deployArgs = [
        'deploy',
        '--prebuilt',
        '--cwd',
        projectPath
    ];
    if (options.debug) {
        buildArgs.push('--debug');
        deployArgs.push('--debug');
    }
    if (options.prod) {
        buildArgs.push('--prod');
        deployArgs.push('--prod');
    }
    try {
        /**
     * The Vercel build command seem to rely on the .vercel folder being in the rootDirectory or at least an ancestor
     * of it, so we temporarily symlink it.
     *
     * NOTE: this means this executor cannot run in parallel on the same machine
     */ // `mklink /D` requires admin privilege in Windows so we need to use junction
        const symlinkType = (0, _os.platform)() === 'win32' ? 'junction' : 'dir';
        await fs.symlink(vercelDir, rootVercelDir, symlinkType);
        await (0, _exec.exec)(vercelCommand, buildArgs, {
            cwd: context.root
        });
    } finally{
        try {
            fs.unlink(rootVercelDir);
            _devkit.logger.debug(`Removed root .vercel symlink`);
        } catch (e) {
            _devkit.logger.warn('Failed to clean up root .vercel symlink');
        }
    }
    if (process.env.VERCEL_TOKEN) {
        _devkit.logger.debug('Detected VERCEL_TOKEN, passing token to Vercel CLI');
        deployArgs.push('--token', process.env.VERCEL_TOKEN);
    }
    /**
   * The deploy command looks for the .git folder next to .vercel, so it can extract metadata. However even if we send
   * correct metadata their API will ignore it if Git integration is not enabled.
   */ const { stdout: deploymentUrl , stderr  } = await (0, _exec.getExecOutput)(vercelCommand, deployArgs, {
        cwd: context.root,
        /**
       * the output gets a bit mangled, as the stdout is inserted into the middle of the regular stderr output, so we
       * silence the stdout and log it ourselves afterwards
       */ silent: true,
        listeners: {
            errline: (line)=>_devkit.logger.log(line)
        }
    });
    _devkit.logger.log(`Deployment: ${deploymentUrl}`);
    var ref;
    const [inspectUrl] = (ref = stderr.match(/(https:\/\/vercel\.com\S+)/i)) != null ? ref : [];
    return {
        success: true,
        metadata: {
            inspectUrl,
            deploymentUrl
        },
        env: {
            VERCEL_INSPECT_URL: inspectUrl,
            VERCEL_DEPLOYMENT_URL: deploymentUrl
        }
    };
};
const _default = (0, _postTargets.withPostTargets)(runExecutor);

//# sourceMappingURL=executor.js.map